#!/usr/bin/env python3

import objc, zipfile, re, subprocess
from AppKit import *
from Quartz.CoreGraphics import *

PICEXT = '\.(jpe?g|png|gif)$'

class ComipoliArchive:
    def __init__(self):
        self.status = 0
        self.namelist = []
        self.path = ''
        self.pdfdoc = None
        self.pdf_max_page = 0

    def new_archive(self, path):
        self.path = path
        if re.search(r'\.(cbz|zip)$', path, re.I):
            self.status = 0
            self.namelist.clear()
            self.pdfdoc = None
            with zipfile.ZipFile(path) as o:
                l = o.namelist()
                l.sort()
                for name in l:
                    if re.search(PICEXT, name, re.I):
                        self.namelist.append(name)
        elif NSApp.path_rar != '' and re.search(r'\.(cbr|rar)$', path, re.I):
            self.status = 1
            self.namelist.clear()
            self.pdfdoc = None
            cp = subprocess.run(['/Users/sasakima-nao/.local/bin/unrar', 'vt', '-p-', '--', path], stdout=subprocess.PIPE)
            lines = cp.stdout.decode('utf-8').split('\n')
            for line in lines:
                s = line.lstrip()
                if s.startswith('Name: '):
                    name = s[6:]
                    if re.search(PICEXT, name, re.I):
                        self.namelist.append(name)
        elif NSApp.path_7za != '' and re.search(r'\.(cb7|7z)$', path, re.I):
            self.status = 2
            self.namelist.clear()
            self.pdfdoc = None
            cp = subprocess.run(['/usr/bin/env', '7za', 'l', '-slt', path], stdout=subprocess.PIPE)
            lines = cp.stdout.decode('utf-8').split('\n')
            for line in lines:
                if line.startswith('Path'):
                    name = line[7:]
                    if re.search(PICEXT, name, re.I):
                        self.namelist.append(name)
        elif NSApp.is_pdf and re.search(r'\.pdf$', path, re.I):
            self.status = 3
            self.namelist.clear()
            url = NSURL.fileURLWithPath_isDirectory_(path, False)
            self.pdfdoc = CGPDFDocumentCreateWithURL(url)
            self.pdf_max_page = CGPDFDocumentGetNumberOfPages(self.pdfdoc)
        else:
            return False
        return True

    def __getitem__(self, num):
        if self.status == 0:
            with zipfile.ZipFile(self.path) as o:
                b = o.read(self.namelist[num])
                data = NSData.dataWithBytes_length_(b, len(b))
                return NSImage.alloc().initWithData_(data)
        elif self.status == 1:
            cp = subprocess.run([NSApp.path_rar, 'p', '-inul', '-@', '--', self.path, self.namelist[num]], stdout=subprocess.PIPE)
            data = NSData.dataWithBytes_length_(cp.stdout, len(cp.stdout))
            return NSImage.alloc().initWithData_(data)
        elif self.status == 2:
            cp = subprocess.run([NSApp.path_7za, 'x', '-so', self.path, self.namelist[num]], stdout=subprocess.PIPE)
            data = NSData.dataWithBytes_length_(cp.stdout, len(cp.stdout))
            return NSImage.alloc().initWithData_(data)
        elif self.status == 3:
            return CGPDFDocumentGetPage(self.pdfdoc, num + 1)

    def __len__(self):
        if self.status == 3:
            return self.pdf_max_page
        return len(self.namelist)

    def get_thumb_image(self, num, aw, ah):
        image = self[num]
        new_image = NSImage.alloc().initWithSize_(NSMakeSize(aw, ah))
        if self.status == 3:
            new_image.lockFocus()
            ctx = NSGraphicsContext.currentContext().CGContext()
            r = CGPDFPageGetBoxRect(image, kCGPDFMediaBox)
            #if r.size.width - r.size.height > 0 or not self.spread:
            CGContextSaveGState(ctx)
            if aw * r.size.height > ah * r.size.width:
                n = ah / r.size.height
                matrix = CGAffineTransformMake(n, 0, 0, n, aw / 2 - r.size.width * n / 2, 0)
            else:
                n = aw / r.size.width
                matrix = CGAffineTransformMake(n, 0, 0, n, n, ah / 2 - r.size.height / 2)
            CGContextConcatCTM(ctx, matrix)
            NSColor.whiteColor().set()
            CGContextFillRect(ctx, r)
            CGContextClipToRect(ctx, r)
            CGContextDrawPDFPage(ctx, image)
            CGContextRestoreGState(ctx)
            new_image.unlockFocus()
            return new_image
        else:
            w = image.size().width
            h = image.size().height
            if aw * h > ah * w:
                width = w * ah / h
                height = ah
                x = (aw - width) / 2
                y = 0
            else:
                width = aw
                height = h * aw / w
                x = 0
                y = (ah - height) / 2
            rect = NSMakeRect(x, y, width, height)
            new_image.lockFocus()
            NSGraphicsContext.saveGraphicsState()
            NSGraphicsContext.currentContext().setImageInterpolation_(NSImageInterpolationHigh)
            image.drawInRect_(rect)
            NSGraphicsContext.restoreGraphicsState()
            new_image.unlockFocus()
            return new_image
