#!/usr/bin/env python3

import objc
from AppKit import *

class ItemView(NSView):
    def init(self):
        objc.super(ItemView, self).init()
        self.image = None
        self.num = 0
        return self

    def drawRect_(self, rect):
        NSColor.darkGrayColor().set()
        NSRectFill(rect)
        if (self.image):
            r = NSMakeRect(5, 5, rect.size.width - 5, rect.size.height - 5)
            self.image.drawInRect_(r)
            bg_color = NSColor.colorWithSRGBRed_green_blue_alpha_(0.4, 1.0, 1.0, 0.5)
            text = NSAttributedString.alloc().initWithString_attributes_(
                '{}'.format(self.num + 1),
                {
                    NSForegroundColorAttributeName: NSColor.redColor(),
                    NSBackgroundColorAttributeName: bg_color
                })
            x = rect.size.width / 2
            NSColor.blueColor().set()
            text.drawAtPoint_(NSMakePoint(x, 5))

    def mouseDown_(self, event):
        w = NSApp.keyWindow().owner
        w.num = self.num - 1
        w.change_pixbuf(1)
        NSApp.keyWindow().close()

class CollectionViewItem(NSCollectionViewItem):
    def init(self):
        objc.super(CollectionViewItem, self).init()
        return self

    def setRepresentedObject_(self, rep):
        objc.super(CollectionViewItem, self).setRepresentedObject_(rep)
        if rep != None:
            self.view().image = rep['pic']
            self.view().num = rep['num']
            #self.view().select = self.isSelected()

class ComipoliCollectionView(NSCollectionView):
    def init(self):
        objc.super(ComipoliCollectionView, self).init()
        self.owner = None
        return self

    def drawRect_(self, rect):
        NSColor.darkGrayColor().set()
        NSRectFill(rect)

class ComipoliCollection(NSWindow):

    avoid_gc = []

    def initWithFrame_pSize_atOwner_(self, frame, psize, owner):
        objc.super(ComipoliCollection, self).initWithContentRect_styleMask_backing_defer_(
            frame,
            NSTitledWindowMask |
            NSClosableWindowMask |
            NSResizableWindowMask,
            NSBackingStoreBuffered,
            False)
        self.setTitle_('Comipoli Thumbnail')
        self.setDelegate_(self)
        self.owner = owner
        # ScrollView
        self.scroll = NSScrollView.new()
        self.scroll.setFrameSize_(self.contentView().frame().size)
        self.scroll.setHasVerticalScroller_(True)
        self.scroll.setFrameSize_(frame.size)
        # ItemView
        self.item = ItemView.alloc().initWithFrame_(NSMakeRect(0, 0, psize + 10, psize + 10))
        # CollectionViewItem
        view_item = CollectionViewItem.new()
        view_item.setView_(self.item)
        # CollectionView
        self.collection = ComipoliCollectionView.new()
        self.collection.setSelectable_(True)
        self.collection.setWantsLayer_(True)
        self.collection.setItemPrototype_(view_item)
        self.collection.setFrameSize_(frame.size)
        self.collection.owner = self
        # combine
        self.scroll.setDocumentView_(self.collection)
        self.contentView().addSubview_(self.scroll)
        return self

    @classmethod
    def newWithFrame_pSize_atOwner_(self, frame, height, owner):
        window = ComipoliCollection.alloc().initWithFrame_pSize_atOwner_(frame, height, owner)
        self.avoid_gc.append(window)
        window.makeKeyAndOrderFront_(window)
        return window

    @objc.python_method
    def set_array(self, array):
        self.collection.setContent_(array)

    def cancelOperation_(self, sender):
        self.close()

    def windowDidResize_(self, sender):
        self.scroll.setFrameSize_(self.contentView().frame().size)

    def windowWillClose_(self, sender):
        NSApp.stopModal()
