#!/usr/bin/env python3

import objc
from AppKit import *
from comipoli_window import ComipoliWindow
from comipoli_preference import ComipoliPreference

class ComipoliMenu(NSMenu):
    def init(self):
        objc.super(ComipoliMenu, self).init()
        # Menubar
        itemApp  = NSMenuItem.new()
        itemFile = NSMenuItem.new()
        itemPage = NSMenuItem.new()
        itemView = NSMenuItem.new()
        itemWind = NSMenuItem.new()
        self.addItem_(itemApp)
        self.addItem_(itemFile)
        self.addItem_(itemPage)        
        self.addItem_(itemView)        
        self.addItem_(itemWind)        
        # Drop Down Menu
        menuApp   = NSMenu.new()
        menuFile  = NSMenu.alloc().initWithTitle_('File')
        menuPage  = NSMenu.alloc().initWithTitle_('Page')        
        menuView  = NSMenu.alloc().initWithTitle_('View')        
        menuWind  = NSMenu.alloc().initWithTitle_('Window')
        itemApp.setSubmenu_(menuApp)
        itemFile.setSubmenu_(menuFile)
        itemPage.setSubmenu_(menuPage)
        itemView.setSubmenu_(menuView)
        itemWind.setSubmenu_(menuWind)
        # Action
        menuApp.addItem_(self._menu('Preference', 'preference:', ',', self, 0))
        menuApp.addItem_(self._menu('About Comipoli', 'aboutDialog:', '', self, 0))
        menuApp.addItem_(NSMenuItem.separatorItem())
        menuApp.addItem_(self._menu('Hide', 'hide:', 'h', None, 0))
        menuApp.addItem_(self._menu('Hide Others', 'hideOtherApplications:', 'h', None, 3))
        menuApp.addItem_(self._menu('Show All', 'unhideAllApplications:', '', None, 0))
        menuApp.addItem_(NSMenuItem.separatorItem())
        menuApp.addItem_(self._menu('Quit Comipoli', 'terminate:', 'q', None, 0))
        # File
        menuFile.addItem_(self._menu('New Window', 'newWindow:', 'n', self, 0))
        menuFile.addItem_(NSMenuItem.separatorItem())        
        menuFile.addItem_(self._menu('Close Window', 'closeWindow:', 'w', self, 0))
        menuFile.addItem_(NSMenuItem.separatorItem())        
        menuFile.addItem_(self._menu('Open', 'openFile:', 'o', self, 0))
        # Page
        menuPage.addItem_(self._menu('Next Page', 'nextPage:', NSLeftArrowFunctionKey, self, 1))
        menuPage.addItem_(self._menu('Step Next Page', 'stepNextPage:', NSLeftArrowFunctionKey, self, 2))
        menuPage.addItem_(self._menu('Previos Page', 'prevPage:', NSRightArrowFunctionKey, self, 1))
        menuPage.addItem_(self._menu('Step Previos Page', 'stepPrevPage:', NSRightArrowFunctionKey, self, 2))
        menuPage.addItem_(NSMenuItem.separatorItem())
        menuPage.addItem_(self._menu('Go First Page', 'firstPage:', NSRightArrowFunctionKey, self, 0))
        menuPage.addItem_(self._menu('Go Last Page', 'lastPage:', NSLeftArrowFunctionKey, self, 0))
        # View
        menuView.addItem_(self._menu('Toggle Full Screen', 'fullScreen:', 'f', self, 4))
        menuView.addItem_(NSMenuItem.separatorItem())
        menuView.addItem_(self._menu('Change L/R', 'changeLR:', 'r', self, 0))
        menuView.addItem_(self._menu('Show Thumbnail', 'showThumbnail:', 'l', self, 0))
        # Window
        menuWind.addItem_(self._menu('Minimize', 'miniaturizeAll:', 'm', None, 0))
        #
        return self

    def preference_(self, sender):
        ComipoliPreference.newWindow()

    def aboutDialog_(self, sender):
        options = {
            'Version': '0.0.2',
            'ApplicationName': 'Comipoli',
            'Copyright': 'Copyright(C) 2018-2019, sasakima-nao',
            'ApplicationVersion': 'Version 0.0.2'
        }
        NSApp.orderFrontStandardAboutPanelWithOptions_(options)

    def openFile_(self, sender):
        NSApp.keyWindow().onOpenButtonClicked_('/')

    def newWindow_(self, sender):
        ComipoliWindow.newWindow()

    def closeWindow_(self, sender):
        NSApp.keyWindow().close()

    def nextPage_(self, sender):
        NSApp.keyWindow().change_pixbuf(2)

    def stepNextPage_(self, sender):
        NSApp.keyWindow().change_pixbuf(1)

    def prevPage_(self, sender):
        NSApp.keyWindow().change_pixbuf(-2)

    def stepPrevPage_(self, sender):
        NSApp.keyWindow().change_pixbuf(-1)

    def firstPage_(self, sender):
        NSApp.keyWindow().change_pixbuf(0)

    def lastPage_(self, sender):
        NSApp.keyWindow().change_pixbuf(50)

    def fullScreen_(self, sender):
        NSApp.keyWindow().toggle_full_screen()

    def changeLR_(self, sender):
        NSApp.keyWindow().onLRButtonClicked_(None)

    def showThumbnail_(self, sender):
        NSApp.keyWindow().onThumbButtonClicked_(None)

    @objc.python_method
    def _menu(self, title, action, key, target, flag):
        item = NSMenuItem.new()
        if target:
            item.setTarget_(target)
        item.setTitle_(title)
        item.setAction_(action)
        item.setKeyEquivalent_(key)
        if flag == 1:
            item.setKeyEquivalentModifierMask_(0)
        elif flag == 2:
            item.setKeyEquivalentModifierMask_(NSEventModifierFlagOption)
        elif flag == 3:
            item.setKeyEquivalentModifierMask_(NSEventModifierFlagCommand | NSEventModifierFlagOption)
        elif flag == 4:
            item.setKeyEquivalentModifierMask_(NSEventModifierFlagCommand | NSEventModifierFlagControl)
        return item
