#!/usr/bin/env python3

import objc
from AppKit import *

class ComipoliPreference(NSWindow):

    avoid_gc = []

    def init(self):
        objc.super(ComipoliPreference, self).initWithContentRect_styleMask_backing_defer_(
            NSMakeRect(0, 0, 340, 180),
            NSTitledWindowMask |
            NSClosableWindowMask,
            NSBackingStoreBuffered,
            False)
        point = NSApp.keyWindow().cascadeTopLeftFromPoint_(NSZeroPoint)
        self.setFrameTopLeftPoint_(point)
        self.setTitle_('Comipoli Preference')
        self.setDelegate_(self)
        # pdf
        self.pdf = NSButton.checkboxWithTitle_target_action_('Enable PDF File', self, self.onEvent_)
        self.pdf.setFrameOrigin_((20, 150))
        self.contentView().addSubview_(self.pdf)
        if NSApp.is_pdf:
            self.pdf.setState_(1)
        # label
        self.label = NSTextField.alloc().initWithFrame_(NSMakeRect(10, 10, 300, 20))
        self.label.setDrawsBackground_(False)
        self.label.setBordered_(False)
        self.label.setEditable_(False)
        self.label.setSelectable_(False)
        self.label.setFrameOrigin_((20, 110))
        text = 'Thumbnail Height ({} px)'.format(NSApp.thumbnail_height)
        self.label.setStringValue_(text)
        self.contentView().addSubview_(self.label)
        # slider
        self.slider = NSSlider.sliderWithValue_minValue_maxValue_target_action_(
            NSApp.thumbnail_height, 80, 200, self, self.onSliderEvent_)
        self.slider.setFrameOrigin_((20, 90))
        self.slider.setNumberOfTickMarks_(7)
        self.slider.setAllowsTickMarkValuesOnly_(True)
        self.contentView().addSubview_(self.slider)
        # rar, 7za
        self.textrar = NSTextField.alloc().initWithFrame_(NSMakeRect(20, 50, 300, 20))
        self.text7za = NSTextField.alloc().initWithFrame_(NSMakeRect(20, 20, 300, 20))
        self.textrar.setPlaceholderString_('unrar Full Path')
        self.text7za.setPlaceholderString_('7za Full Path')
        self.textrar.setStringValue_(NSApp.path_rar)
        self.text7za.setStringValue_(NSApp.path_7za)
        self.contentView().addSubview_(self.textrar)
        self.contentView().addSubview_(self.text7za)
        #
        return self

    def cancelOperation_(self, sender):
        self.close()

    def onEvent_(self, sender):
        pass

    def onSliderEvent_(self, sender):
        text = 'Thumbnail Height ({} px)'.format(sender.floatValue())
        self.label.setStringValue_(text)

    def windowWillClose_(self, sender):
        NSApp.is_pdf = self.pdf.state() == NSOnState
        NSApp.thumbnail_height = self.slider.floatValue()
        NSApp.path_rar = self.textrar.stringValue()

    @classmethod
    def newWindow(self):
        window = ComipoliPreference.new()
        self.avoid_gc.append(window)
        window.makeKeyAndOrderFront_(window)
        return window
