#!/usr/bin/env python3

import objc
from AppKit import *
from Quartz.CoreGraphics import *

class ComipoliView(NSView):
    def init(self):
        objc.super(ComipoliView, self).init()
        self.first_page = None
        self.second_page = None
        self.spread = False
        self.l_to_r = False
        self.is_pdf = False
        return self

    def acceptsFirstResponder(self):
        return True

    def keyDown_(self, event):
        window = NSApp.keyWindow()
        if window.is_fullscreen:
            window.toggle_full_screen()
        elif window.esc_to_close:
            window.close()

    def drawRect_(self, rect):
        NSColor.blackColor().set()
        NSRectFill(rect)
        NSColor.whiteColor().set()
        if self.first_page != None:
            if self.is_pdf:
                ctx = NSGraphicsContext.currentContext().CGContext()
                r = CGPDFPageGetBoxRect(self.first_page, kCGPDFMediaBox)
                if r.size.width - r.size.height > 0 or not self.spread:
                    CGContextSaveGState(ctx)
                    if rect.size.width * r.size.height > rect.size.height * r.size.width:
                        n = rect.size.height / r.size.height
                        matrix = CGAffineTransformMake(n, 0, 0, n, rect.size.width / 2 - r.size.width * n / 2, 0)
                    else:
                        n = rect.size.width / r.size.width
                        matrix = CGAffineTransformMake(n, 0, 0, n, n, rect.size.height / 2 - r.size.height / 2)
                    CGContextConcatCTM(ctx, matrix)
                    NSColor.whiteColor().set()
                    CGContextFillRect(ctx, r)
                    CGContextClipToRect(ctx, r)
                    CGContextDrawPDFPage(ctx, self.first_page)
                    CGContextRestoreGState(ctx)
                elif self.second_page != None:
                    if self.l_to_r:
                        CGContextSaveGState(ctx)
                        n = rect.size.height / r.size.height
                        matrix = CGAffineTransformMake(n, 0, 0, n, rect.size.width / 2 - r.size.width * n, n)
                        CGContextConcatCTM(ctx, matrix)
                        NSColor.whiteColor().set()
                        CGContextFillRect(ctx, r)
                        CGContextClipToRect(ctx, r)
                        CGContextDrawPDFPage(ctx, self.first_page)
                        CGContextRestoreGState(ctx)
                        #
                        CGContextSaveGState(ctx)
                        r = CGPDFPageGetBoxRect(self.second_page, kCGPDFMediaBox)
                        n = rect.size.height / r.size.height
                        matrix = CGAffineTransformMake(n, 0, 0, n, rect.size.width / 2, n)
                        CGContextConcatCTM(ctx, matrix)
                        NSColor.whiteColor().set()
                        CGContextFillRect(ctx, r)
                        CGContextClipToRect(ctx, r)
                        CGContextDrawPDFPage(ctx, self.second_page)
                        CGContextRestoreGState(ctx)
                    else:
                        CGContextSaveGState(ctx)
                        n = rect.size.height / r.size.height
                        matrix = CGAffineTransformMake(n, 0, 0, n, rect.size.width / 2, n)
                        CGContextConcatCTM(ctx, matrix)
                        NSColor.whiteColor().set()
                        CGContextFillRect(ctx, r)
                        CGContextClipToRect(ctx, r)
                        CGContextDrawPDFPage(ctx, self.first_page)
                        CGContextRestoreGState(ctx)
                        #
                        CGContextSaveGState(ctx)
                        r = CGPDFPageGetBoxRect(self.second_page, kCGPDFMediaBox)
                        n = rect.size.height / r.size.height
                        matrix = CGAffineTransformMake(n, 0, 0, n, rect.size.width / 2 - r.size.width * n, n)
                        CGContextConcatCTM(ctx, matrix)
                        NSColor.whiteColor().set()
                        CGContextFillRect(ctx, r)
                        CGContextClipToRect(ctx, r)
                        CGContextDrawPDFPage(ctx, self.second_page)
                        CGContextRestoreGState(ctx)
            else:
                aw = rect.size.width
                ah = rect.size.height
                w = self.first_page.size().width
                h = self.first_page.size().height
                # Horizontal?
                if w - h > 0 or not self.spread:
                    # Single Page
                    if aw * h > ah * w:
                        width = w * ah / h
                        height = ah
                        x = (aw - width) / 2
                        y = 0
                    else:
                        width = aw
                        height = h * aw / w
                        x = 0
                        y = (ah - height) / 2
                    r1 = NSMakeRect(x, y, width, height)
                    self.first_page.drawInRect_(r1)
                elif self.second_page != None:
                    left = ah * w / h
                    if self.l_to_r:
                        r1 = NSMakeRect(aw / 2 - left, 0, left, ah)
                        self.first_page.drawInRect_(r1)
                        #
                        w2 = self.second_page.size().width
                        h2 = self.second_page.size().height
                        right = ah * w2 / h2
                        r2 = NSMakeRect(aw / 2, 0, right, ah)
                        self.second_page.drawInRect_(r2)
                    else:
                        r1 = NSMakeRect(aw / 2, 0, left, ah)
                        NSRectFill(r1)
                        self.first_page.drawInRect_(r1)
                        #
                        w2 = self.second_page.size().width
                        h2 = self.second_page.size().height
                        right = ah * w2 / h2
                        r2 = NSMakeRect(aw / 2 - right, 0, right, ah)
                        self.second_page.drawInRect_(r2)
